"use strict";
var Node = require('./node');
var Declaration = require('./declaration');
var Container = function Container() {
  $traceurRuntime.defaultSuperCall(this, $Container.prototype, arguments);
};
var $Container = Container;
($traceurRuntime.createClass)(Container, {
  stringifyContent: function(builder) {
    if (!this.rules && !this.decls)
      return;
    var i,
        last = this.list.length - 1;
    if (this.rules) {
      for (i = 0; i < this.rules.length; i++) {
        this.rules[i].stringify(builder, last == i);
      }
    } else if (this.decls) {
      for (i = 0; i < this.decls.length; i++) {
        this.decls[i].stringify(builder, last != i || this.semicolon);
      }
    }
  },
  defaultAfter: function() {
    if (this.list.length === 0) {
      return '';
    } else {
      var last = this.list[0].before;
      if (typeof(last) != 'undefined' && last.indexOf("\n") == -1) {
        return this.list[0].before;
      } else {
        return "\n";
      }
    }
  },
  stringifyBlock: function(builder, start) {
    var style = this.style();
    if (this.before)
      builder(this.before);
    builder(start, this, 'start');
    this.stringifyContent(builder);
    if (style.after)
      builder(style.after);
    builder('}', this, 'end');
  },
  push: function(child) {
    child.parent = this;
    this.list.push(child);
    return this;
  },
  each: function(callback) {
    if (!this.lastEach)
      this.lastEach = 0;
    if (!this.indexes)
      this.indexes = {};
    this.lastEach += 1;
    var id = this.lastEach;
    this.indexes[id] = 0;
    var list = this.list;
    if (!list)
      return;
    var index,
        result;
    while (this.indexes[id] < list.length) {
      index = this.indexes[id];
      result = callback(list[index], index);
      if (result === false)
        break;
      this.indexes[id] += 1;
    }
    delete this.indexes[id];
    if (result === false)
      return false;
  },
  eachInside: function(callback) {
    return this.each((function(child, i) {
      var result = callback(child, i);
      if (result !== false && child.eachInside) {
        result = child.eachInside(callback);
      }
      if (result === false)
        return result;
    }));
  },
  eachDecl: function(callback) {},
  eachComment: function(callback) {
    return this.eachInside((function(child, i) {
      if (child.type == 'comment') {
        var result = callback(child, i);
        if (result === false)
          return result;
      }
    }));
  },
  append: function(child) {
    var childs = this.normalize(child, this.list[this.list.length - 1]);
    for (var $__2 = childs[Symbol.iterator](),
        $__3; !($__3 = $__2.next()).done; ) {
      child = $__3.value;
      {
        this.list.push(child);
      }
    }
    return this;
  },
  prepend: function(child) {
    var childs = this.normalize(child, this.list[0], 'prepend').reverse();
    for (var $__2 = childs[Symbol.iterator](),
        $__3; !($__3 = $__2.next()).done; ) {
      child = $__3.value;
      {
        this.list.unshift(child);
      }
    }
    for (var id in this.indexes) {
      this.indexes[id] = this.indexes[id] + childs.length;
    }
    return this;
  },
  insertBefore: function(exist, add) {
    exist = this.index(exist);
    var type = exist === 0 ? 'prepend' : false;
    var childs = this.normalize(add, this.list[exist], type).reverse();
    for (var $__2 = childs[Symbol.iterator](),
        $__3; !($__3 = $__2.next()).done; ) {
      var child = $__3.value;
      {
        this.list.splice(exist, 0, child);
      }
    }
    for (var id in this.indexes) {
      this.indexes[id] = this.indexes[id] + childs.length;
    }
    return this;
  },
  insertAfter: function(exist, add) {
    exist = this.index(exist);
    var childs = this.normalize(add, this.list[exist]).reverse();
    for (var $__2 = childs[Symbol.iterator](),
        $__3; !($__3 = $__2.next()).done; ) {
      var child = $__3.value;
      {
        this.list.splice(exist + 1, 0, child);
      }
    }
    for (var id in this.indexes) {
      this.indexes[id] = this.indexes[id] + childs.length;
    }
    return this;
  },
  remove: function(child) {
    child = this.index(child);
    this.list.splice(child, 1);
    for (var id in this.indexes) {
      var index = this.indexes[id];
      if (index >= child) {
        this.indexes[id] = index - 1;
      }
    }
    return this;
  },
  every: function(condition) {
    return this.list.every(condition);
  },
  some: function(condition) {
    return this.list.some(condition);
  },
  index: function(child) {
    if (typeof(child) == 'number') {
      return child;
    } else {
      return this.list.indexOf(child);
    }
  },
  get first() {
    if (!this.list)
      return undefined;
    return this.list[0];
  },
  get last() {
    if (!this.list)
      return undefined;
    return this.list[this.list.length - 1];
  },
  get list() {
    return this.rules || this.decls;
  },
  normalize: function(child, sample) {
    if (child.type == 'root') {
      return this.normalize(child.rules, sample);
    } else {
      var childs;
      if (Array.isArray(child)) {
        childs = child.map((function(i) {
          return i.clone();
        }));
      } else {
        childs = [child];
      }
      for (var $__2 = childs[Symbol.iterator](),
          $__3; !($__3 = $__2.next()).done; ) {
        child = $__3.value;
        {
          child.parent = this;
          if (typeof(child.before) == 'undefined' && sample) {
            child.before = sample.before;
          }
        }
      }
      return childs;
    }
  }
}, {}, Node);
Container.WithRules = (function($__super) {
  var $__0 = function(defaults) {
    this.rules = [];
    $traceurRuntime.superCall(this, $__0.prototype, "constructor", [defaults]);
  };
  return ($traceurRuntime.createClass)($__0, {
    eachDecl: function(callback) {
      return this.each((function(child) {
        if (child.eachDecl) {
          var result = child.eachDecl(callback);
          if (result === false)
            return result;
        }
      }));
    },
    eachRule: function(callback) {
      return this.each((function(child, i) {
        var result;
        if (child.type == 'rule') {
          result = callback(child, i);
        } else if (child.eachRule) {
          result = child.eachRule(callback);
        }
        if (result === false)
          return result;
      }));
    },
    eachAtRule: function(callback) {
      return this.eachInside((function(child, i) {
        if (child.type == 'atrule') {
          var result = callback(child, i);
          if (result === false)
            return result;
        }
      }));
    }
  }, {}, $__super);
}(Container));
Container.WithDecls = (function($__super) {
  var $__0 = function(defaults) {
    this.decls = [];
    $traceurRuntime.superCall(this, $__0.prototype, "constructor", [defaults]);
  };
  return ($traceurRuntime.createClass)($__0, {
    normalize: function(child, sample, type) {
      if (!child.type && !Array.isArray(child)) {
        child = new Declaration(child);
      }
      return $traceurRuntime.superCall(this, $__0.prototype, "normalize", [child, sample, type]);
    },
    eachDecl: function(callback) {
      return this.each((function(child, i) {
        if (child.type == 'decl') {
          var result = callback(child, i);
          if (result === false)
            return result;
        }
      }));
    }
  }, {}, $__super);
}(Container));
module.exports = Container;
