"use strict";
var Result = require('./result');
var base64js = require('base64-js');
var mozilla = require('source-map');
var path = require('path');
var MapGenerator = function MapGenerator(root, opts) {
  this.root = root;
  this.opts = opts;
  this.mapOpts = opts.map || {};
};
($traceurRuntime.createClass)(MapGenerator, {
  isMap: function() {
    if (typeof(this.opts.map) != 'undefined') {
      return !!this.opts.map;
    } else {
      return this.previous().length > 0;
    }
  },
  previous: function() {
    var $__0 = this;
    if (!this.previousMaps) {
      this.previousMaps = [];
      this.root.eachInside((function(node) {
        if (node.source && node.source.map) {
          if ($__0.previousMaps.indexOf(node.source.map) == -1) {
            $__0.previousMaps.push(node.source.map);
          }
        }
      }));
    }
    return this.previousMaps;
  },
  isInline: function() {
    if (typeof(this.mapOpts.inline) != 'undefined') {
      return this.mapOpts.inline;
    }
    return this.previous().some((function(i) {
      return i.inline;
    }));
  },
  isSourcesContent: function() {
    if (typeof(this.mapOpts.sourcesContent) != 'undefined') {
      return this.mapOpts.sourcesContent;
    }
    return this.previous().some((function(i) {
      return i.withContent();
    }));
  },
  clearAnnotation: function() {
    var last = this.root.last;
    if (!last)
      return;
    if (last.type != 'comment')
      return;
    if (last.text.match(/^# sourceMappingURL=/)) {
      last.removeSelf();
    }
  },
  setSourcesContent: function() {
    var $__0 = this;
    var already = {};
    this.root.eachInside((function(node) {
      if (node.source && !already[node.source.file]) {
        already[node.source.file] = true;
        var relative = $__0.relative(node.source.file);
        $__0.map.setSourceContent(relative, node.source.content);
      }
    }));
  },
  applyPrevMaps: function() {
    for (var $__2 = this.previous()[Symbol.iterator](),
        $__3; !($__3 = $__2.next()).done; ) {
      var prev = $__3.value;
      {
        var from = this.relative(prev.file);
        var root = prev.root || path.dirname(prev.file);
        var map;
        if (this.mapOpts.sourcesContent === false) {
          map = new mozilla.SourceMapConsumer(prev.text);
          map.sourcesContent = map.sourcesContent.map((function(i) {
            return null;
          }));
        } else {
          map = prev.consumer();
        }
        this.map.applySourceMap(map, from, this.relative(root));
      }
    }
  },
  isAnnotation: function() {
    if (this.isInline()) {
      return true;
    } else if (typeof(this.mapOpts.annotation) != 'undefined') {
      return this.mapOpts.annotation;
    } else if (this.previous().length) {
      return this.previous().some((function(i) {
        return i.annotation;
      }));
    } else {
      return true;
    }
  },
  addAnnotation: function() {
    var content;
    if (this.isInline()) {
      var string = this.map.toString();
      var bytes = [];
      for (var i = 0; i < string.length; i++) {
        bytes.push(string.charCodeAt(i));
      }
      content = "data:application/json;base64," + base64js.fromByteArray(bytes);
    } else if (typeof(this.mapOpts.annotation) == 'string') {
      content = this.mapOpts.annotation;
    } else {
      content = this.outputFile() + '.map';
    }
    this.css += "\n/*# sourceMappingURL=" + content + " */";
  },
  outputFile: function() {
    return this.opts.to ? this.relative(this.opts.to) : 'to.css';
  },
  generateMap: function() {
    this.stringify();
    if (this.isSourcesContent())
      this.setSourcesContent();
    if (this.previous().length > 0)
      this.applyPrevMaps();
    if (this.isAnnotation())
      this.addAnnotation();
    if (this.isInline()) {
      return [this.css];
    } else {
      return [this.css, this.map];
    }
  },
  relative: function(file) {
    var from = this.opts.to ? path.dirname(this.opts.to) : '.';
    if (typeof(this.mapOpts.annotation) == 'string') {
      from = path.dirname(path.resolve(from, this.mapOpts.annotation));
    }
    file = path.relative(from, file);
    if (path.sep == '\\') {
      return file.replace('\\', '/');
    } else {
      return file;
    }
  },
  sourcePath: function(node) {
    return this.relative(node.source.file || 'from.css');
  },
  stringify: function() {
    var $__0 = this;
    this.css = '';
    this.map = new mozilla.SourceMapGenerator({file: this.outputFile()});
    var line = 1;
    var column = 1;
    var lines,
        last;
    var builder = (function(str, node, type) {
      $__0.css += str;
      if (node && node.source && node.source.start && type != 'end') {
        $__0.map.addMapping({
          source: $__0.sourcePath(node),
          original: {
            line: node.source.start.line,
            column: node.source.start.column - 1
          },
          generated: {
            line: line,
            column: column - 1
          }
        });
      }
      lines = str.match(/\n/g);
      if (lines) {
        line += lines.length;
        last = str.lastIndexOf("\n");
        column = str.length - last;
      } else {
        column = column + str.length;
      }
      if (node && node.source && node.source.end && type != 'start') {
        $__0.map.addMapping({
          source: $__0.sourcePath(node),
          original: {
            line: node.source.end.line,
            column: node.source.end.column
          },
          generated: {
            line: line,
            column: column
          }
        });
      }
    });
    this.root.stringify(builder);
  },
  generate: function() {
    this.clearAnnotation();
    if (this.isMap()) {
      return this.generateMap();
    } else {
      return [this.root.toString()];
    }
  }
}, {});
module.exports = MapGenerator;
