"use strict";
var clone = function(obj, parent) {
  if (typeof(obj) != 'object')
    return obj;
  var cloned = new obj.constructor();
  for (var name in obj) {
    if (!obj.hasOwnProperty(name))
      continue;
    var value = obj[name];
    if (name == 'parent' && typeof(value) == 'object') {
      if (parent)
        cloned[name] = parent;
    } else if (name == 'source') {
      cloned[name] = value;
    } else if (value instanceof Array) {
      cloned[name] = value.map((function(i) {
        return clone(i, cloned);
      }));
    } else {
      cloned[name] = clone(value, cloned);
    }
  }
  return cloned;
};
var keys = function(obj, keys) {
  var all = {};
  for (var key in keys) {
    if (typeof(obj[key]) == 'undefined') {
      return false;
    } else {
      all[key] = obj[key];
    }
  }
  return all;
};
var Node = function Node() {
  var defaults = arguments[0] !== (void 0) ? arguments[0] : {};
  for (var name in defaults) {
    this[name] = defaults[name];
  }
};
($traceurRuntime.createClass)(Node, {
  removeSelf: function() {
    if (this.parent) {
      this.parent.remove(this);
    }
    return this;
  },
  toString: function() {
    var result = '';
    var builder = (function(str) {
      return result += str;
    });
    this.stringify(builder);
    return result;
  },
  clone: function() {
    var overrides = arguments[0] !== (void 0) ? arguments[0] : {};
    var cloned = clone(this);
    for (var name in overrides) {
      cloned[name] = overrides[name];
    }
    return cloned;
  },
  toJSON: function() {
    var fixed = {};
    for (var name in this) {
      if (!this.hasOwnProperty(name))
        continue;
      if (name == 'parent')
        continue;
      var value = this[name];
      if (value instanceof Array) {
        fixed[name] = value.map((function(i) {
          return (typeof(i) == 'object' && i.toJSON) ? i.toJSON() : i;
        }));
      } else if (typeof(value) == 'object' && value.toJSON) {
        fixed[name] = value.toJSON();
      } else {
        fixed[name] = value;
      }
    }
    return fixed;
  },
  defaultStyle: function() {
    return {};
  },
  styleType: function() {
    return this.type;
  },
  style: function() {
    var $__0 = this;
    var type = this.styleType();
    var defaults = this.defaultStyle(type);
    var all = keys(this, defaults);
    if (all)
      return all;
    var styled = defaults;
    if (this.parent) {
      root = this;
      while (root.parent)
        root = root.parent;
      if (!root.styleCache)
        root.styleCache = {};
      if (root.styleCache[type]) {
        styled = root.styleCache[type];
      } else {
        root.eachInside((function(another) {
          if (another.styleType() != type)
            return;
          if ($__0 == another)
            return;
          all = keys(another, styled);
          if (all) {
            styled = all;
            return false;
          }
        }));
        root.styleCache[type] = styled;
      }
    }
    var merge = {};
    for (var key in styled) {
      if (typeof(this[key]) == 'undefined') {
        merge[key] = styled[key];
      } else {
        merge[key] = this[key];
      }
    }
    return merge;
  },
  stringifyRaw: function(prop) {
    var value = this[prop];
    var raw = this['_' + prop];
    if (raw && raw.value === value) {
      return raw.raw;
    } else {
      return value;
    }
  }
}, {});
module.exports = Node;
